@@ -16,6 +16,7 @@
 #include "include/core/SkSurfaceProps.h"
 #include "include/private/SkNoncopyable.h"
 #include "src/core/SkMatrixProvider.h"
+#include "src/core/SkRasterClip.h"
 #include "src/shaders/SkShaderBase.h"
 
 class SkBitmap;
@@ -51,6 +52,8 @@ public:
         return fSurfaceProps;
     }
 
+    SkIRect bounds() const { return SkIRect::MakeWH(this->width(), this->height()); }
+
     /**
      *  Return the bounds of the device in the coordinate space of the root
      *  canvas. The root device will have its top-left at 0,0, but other devices
@@ -58,7 +61,7 @@ public:
      */
     void getGlobalBounds(SkIRect* bounds) const {
         SkASSERT(bounds);
-        SkRect localBounds = SkRect::MakeIWH(this->width(), this->height());
+        SkRect localBounds = SkRect::Make(this->bounds());
         fDeviceToGlobal.mapRect(&localBounds);
         *bounds = localBounds.roundOut();
     }
@@ -145,6 +148,7 @@ public:
 
     // SkMatrixProvider interface:
     bool getLocalToMarker(uint32_t, SkM44* localToMarker) const override;
+    bool localToDeviceHitsPixelCenters() const override { return true; }
 
     const SkMatrixProvider& asMatrixProvider() const { return *this; }
 
@@ -195,6 +199,9 @@ public:
 
     virtual bool android_utils_clipWithStencil() { return false; }
 
+    virtual GrRecordingContext* recordingContext() const { return nullptr; }
+    virtual GrSurfaceDrawContext* surfaceDrawContext() { return nullptr; }
+
 protected:
     enum TileUsage {
         kPossible_TileUsage,    //!< the created device may be drawn tiled
@@ -262,11 +269,10 @@ protected:
                           bool pathIsMutable = false) = 0;
 
     virtual void drawImageRect(const SkImage*, const SkRect* src, const SkRect& dst,
-                               const SkPaint&, SkCanvas::SrcRectConstraint) = 0;
-    virtual void drawImageNine(const SkImage*, const SkIRect& center,
-                               const SkRect& dst, const SkPaint&);
+                               const SkSamplingOptions&, const SkPaint&,
+                               SkCanvas::SrcRectConstraint) = 0;
     virtual void drawImageLattice(const SkImage*, const SkCanvas::Lattice&,
-                                  const SkRect& dst, const SkPaint&);
+                                  const SkRect& dst, SkFilterMode, const SkPaint&);
 
     virtual void drawVertices(const SkVertices*, SkBlendMode, const SkPaint&) = 0;
     virtual void drawShadow(const SkPath&, const SkDrawShadowRec&);
@@ -278,7 +284,8 @@ protected:
 
     // default implementation calls drawPath
     virtual void drawAtlas(const SkImage* atlas, const SkRSXform[], const SkRect[],
-                           const SkColor[], int count, SkBlendMode, const SkPaint&);
+                           const SkColor[], int count, SkBlendMode, const SkSamplingOptions&,
+                           const SkPaint&);
 
     virtual void drawAnnotation(const SkRect&, const char[], SkData*) {}
 
@@ -292,7 +299,8 @@ protected:
     // are all set. If there's a clip region, it will be applied using clipPath().
     virtual void drawEdgeAAImageSet(const SkCanvas::ImageSetEntry[], int count,
                                     const SkPoint dstClips[], const SkMatrix preViewMatrices[],
-                                    const SkPaint& paint, SkCanvas::SrcRectConstraint);
+                                    const SkSamplingOptions&, const SkPaint&,
+                                    SkCanvas::SrcRectConstraint);
 
     void drawGlyphRunRSXform(const SkFont&, const SkGlyphID[], const SkRSXform[], int count,
                              SkPoint origin, const SkPaint& paint);
@@ -307,13 +315,14 @@ protected:
      * from the input device to this device. The provided SkPaint cannot have a mask filter or
      * image filter, and any shader is ignored.
      */
-    virtual void drawDevice(SkBaseDevice*, const SkPaint&);
+    virtual void drawDevice(SkBaseDevice*, const SkSamplingOptions&, const SkPaint&);
 
     /**
      * Draw the special image's subset to this device, subject to the given matrix transform instead
      * of the device's current local to device matrix.
      */
-    virtual void drawSpecial(SkSpecialImage*, const SkMatrix& localToDevice, const SkPaint&);
+    virtual void drawSpecial(SkSpecialImage*, const SkMatrix& localToDevice,
+                             const SkSamplingOptions&, const SkPaint&);
 
     /**
      * Evaluate 'filter' and draw the final output into this device using 'paint'. The 'mapping'
@@ -325,7 +334,7 @@ protected:
      * The final paint must not have an image filter or mask filter set on it; a shader is ignored.
      */
     virtual void drawFilteredImage(const skif::Mapping& mapping, SkSpecialImage* src,
-                                   const SkImageFilter* filter, const SkPaint& paint);
+                                   const SkImageFilter*, const SkSamplingOptions&, const SkPaint&);
 
     virtual sk_sp<SkSpecialImage> makeSpecial(const SkBitmap&);
     virtual sk_sp<SkSpecialImage> makeSpecial(const SkImage*);
@@ -344,8 +353,6 @@ protected:
 
     ///////////////////////////////////////////////////////////////////////////
 
-    virtual GrRecordingContext* recordingContext() const { return nullptr; }
-
     virtual sk_sp<SkSurface> makeSurface(const SkImageInfo&, const SkSurfaceProps&);
     virtual bool onPeekPixels(SkPixmap*) { return false; }
 
@@ -402,11 +409,14 @@ protected:
         return nullptr;
     }
 
+    // SkCanvas uses NoPixelsDevice when onCreateDevice fails; but then it needs to be able to
+    // inspect a layer's device to know if calling drawDevice() later is allowed.
+    virtual bool isNoPixelsDevice() const { return false; }
+
 private:
     friend class SkAndroidFrameworkUtils;
     friend class SkCanvas;
     friend class SkDraw;
-    friend class SkDrawIter;
     friend class SkSurface_Raster;
     friend class DeviceTestingAccess;
 
@@ -423,10 +433,6 @@ private:
 
     virtual bool forceConservativeRasterClip() const { return false; }
 
-    /**
-     * Don't call this!
-     */
-    virtual GrRenderTargetContext* accessRenderTargetContext() { return nullptr; }
 
     // Configure the device's coordinate spaces, specifying both how its device image maps back to
     // the global space (via 'deviceToGlobal') and the initial CTM of the device (via
@@ -444,10 +450,6 @@ private:
         this->setDeviceCoordinateSystem(SkMatrix::I(), globalCTM, x, y);
     }
 
-    /** Causes any deferred drawing to the device to be completed.
-     */
-    virtual void flush() {}
-
     virtual SkImageFilterCache* getImageFilterCache() { return nullptr; }
 
     friend class SkNoPixelsDevice;
@@ -483,52 +485,80 @@ public:
         //SkASSERT(bounds.width() >= 0 && bounds.height() >= 0);
 
         this->setOrigin(SkM44(), bounds.left(), bounds.top());
+        this->resetClipStack();
     }
 
     void resetForNextPicture(const SkIRect& bounds) {
         //SkASSERT(bounds.width() >= 0 && bounds.height() >= 0);
         this->privateResize(bounds.width(), bounds.height());
         this->setOrigin(SkM44(), bounds.left(), bounds.top());
+        this->resetClipStack();
     }
 
 protected:
-    // We don't track the clip at all (for performance), but we have to respond to some queries.
-    // We pretend to be wide-open. We could pretend to always be empty, but that *seems* worse.
-    void onSave() override {}
-    void onRestore() override {}
-    void onClipRect(const SkRect& rect, SkClipOp, bool aa) override {}
-    void onClipRRect(const SkRRect& rrect, SkClipOp, bool aa) override {}
-    void onClipPath(const SkPath& path, SkClipOp, bool aa) override {}
-    void onClipRegion(const SkRegion& deviceRgn, SkClipOp) override {}
-    void onSetDeviceClipRestriction(SkIRect* mutableClipRestriction) override {}
-    bool onClipIsAA() const override { return false; }
-    bool onClipIsWideOpen() const override { return true; }
+    // SkNoPixelsDevice tracks the clip conservatively in order to respond to some queries as
+    // accurately as possible while emphasizing performance
+    void onSave() override;
+    void onRestore() override;
+    void onClipRect(const SkRect& rect, SkClipOp op, bool aa) override;
+    void onClipRRect(const SkRRect& rrect, SkClipOp op, bool aa) override;
+    void onClipPath(const SkPath& path, SkClipOp op, bool aa) override;
+    void onClipRegion(const SkRegion& globalRgn, SkClipOp op) override;
+    void onClipShader(sk_sp<SkShader> shader) override;
+    void onReplaceClip(const SkIRect& rect) override;
+    void onSetDeviceClipRestriction(SkIRect* mutableClipRestriction) override;
+    bool onClipIsAA() const override { return this->clip().isAA(); }
+    bool onClipIsWideOpen() const override {
+        return this->clip().isRect() &&
+               this->onDevClipBounds() == this->bounds();
+    }
     void onAsRgnClip(SkRegion* rgn) const override {
-        rgn->setRect(SkIRect::MakeWH(this->width(), this->height()));
-    }
-    ClipType onGetClipType() const override {
-        return ClipType::kRect;
-    }
-    SkIRect onDevClipBounds() const override {
-        return SkIRect::MakeWH(this->width(), this->height());
+        rgn->setRect(this->onDevClipBounds());
     }
+    ClipType onGetClipType() const override;
+    SkIRect onDevClipBounds() const override { return this->clip().getBounds(); }
 
     void drawPaint(const SkPaint& paint) override {}
     void drawPoints(SkCanvas::PointMode, size_t, const SkPoint[], const SkPaint&) override {}
     void drawImageRect(const SkImage*, const SkRect*, const SkRect&,
-                       const SkPaint&, SkCanvas::SrcRectConstraint) override {}
+                       const SkSamplingOptions&, const SkPaint&,
+                       SkCanvas::SrcRectConstraint) override {}
     void drawRect(const SkRect&, const SkPaint&) override {}
     void drawOval(const SkRect&, const SkPaint&) override {}
     void drawRRect(const SkRRect&, const SkPaint&) override {}
     void drawPath(const SkPath&, const SkPaint&, bool) override {}
-    void drawDevice(SkBaseDevice*, const SkPaint&) override {}
+    void drawDevice(SkBaseDevice*, const SkSamplingOptions&, const SkPaint&) override {}
     void drawGlyphRunList(const SkGlyphRunList& glyphRunList) override {}
     void drawVertices(const SkVertices*, SkBlendMode, const SkPaint&) override {}
 
-    void drawFilteredImage(const skif::Mapping& mapping, SkSpecialImage* src,
-                           const SkImageFilter* filter, const SkPaint& paint) override {}
+    void drawFilteredImage(const skif::Mapping&, SkSpecialImage* src, const SkImageFilter*,
+                           const SkSamplingOptions&, const SkPaint&) override {}
+
+    bool isNoPixelsDevice() const override { return true; }
 
 private:
+struct ClipState {
+        SkConservativeClip fClip;
+        int fDeferredSaveCount = 0;
+
+        ClipState() = default;
+        explicit ClipState(const SkConservativeClip& clip) : fClip(clip) {}
+    };
+
+    const SkConservativeClip& clip() const { return fClipStack.back().fClip; }
+    SkConservativeClip& writableClip();
+
+    void resetClipStack() {
+        fClipStack.reset();
+        fDeviceClipRestriction.setEmpty();
+        ClipState& state = fClipStack.push_back();
+        state.fClip.setRect(this->bounds());
+        state.fClip.setDeviceClipRestriction(&fDeviceClipRestriction);
+    }
+
+    SkIRect fDeviceClipRestriction;
+    SkSTArray<4, ClipState> fClipStack;
+
     using INHERITED = SkBaseDevice;
 };
 
